local addonName = "StrataFix"
StrataFix = {}
local addon = StrataFix 

local frame = CreateFrame("Button", addonName.."HiddenFrame", UIParent)
local revision = tonumber(("$Revision: 18 $"):match("%d+"))

local function chatMsg(msg)
     DEFAULT_CHAT_FRAME:AddMessage(addonName..": "..msg)
end

-- Names of frames that should never be messed with,
-- specifically those containing child frames with a deliberately lower strata
addon.blacklist = {
  ["TargetFrame"] = true, -- Adapt and other unit frame model addons
  ["FocusFrame"] = true,  -- Adapt and other unit frame model addons
  ["TicketStatusFrame"] = true,
  ["CalendarTexturePickerFrame"] = true,
  ["CalendarModalDummy"] = true,
  ["TransmogrifyFrameMouseBlock"] = true,
  ["TransmogrifyModelFrame"] = true,
}
for i=1,10 do
  addon.blacklist["ChatFrame"..i.."EditBox"] = true
end
for i=1,MIRRORTIMER_NUMTIMERS do
  addon.blacklist["MirrorTimer"..i] = true
end

-- References to frames that should never be messed with,
-- specifically those containing child frames with a deliberately lower strata
addon.weak_blacklist = setmetatable({}, {__mode = "k"}) 

-- support for Skinner
hooksecurefunc("LowerFrameLevel", function(frame) addon.weak_blacklist[frame] = true end)
if Skinner and Skinner.skinFrame then
  for obj,_ in pairs(Skinner.skinFrame) do
    addon.weak_blacklist[obj] = true
  end
end
if Skinner and Skinner.sBtn then
  for obj,_ in pairs(Skinner.sBtn) do
    addon.weak_blacklist[obj] = true
  end
end

function addon:UpdateSettings() 
  addon.settings.blacklist = addon.settings.blacklist or {}
  for name,val in pairs(addon.settings.blacklist) do
    addon.blacklist[name] = val
  end
end

SLASH_STRATAFIX1 = "/stratafix"
SlashCmdList["STRATAFIX"] = function(args)
  local cmd, arg = strsplit(" ",args,2)
  cmd = cmd:lower()
  if cmd == "bladd" and arg then
    addon.settings.blacklist[arg] = true
    chatMsg("Added "..arg.." to blacklist")
    addon:UpdateSettings()
  elseif cmd == "bldel" and arg then
    addon.settings.blacklist[arg] = false
    chatMsg("Removed "..arg.." from blacklist")
    addon:UpdateSettings()
  elseif cmd == "blshow" then
    chatMsg("Current frame blacklist:")
    local sorttmp = {}
    for name,val in pairs(addon.blacklist) do
      if val then
        table.insert(sorttmp,name)
      end
    end
    table.sort(sorttmp)
    for _,name in ipairs(sorttmp) do
      chatMsg("  "..name)
    end
  else
    chatMsg("Revision "..revision)
    chatMsg("/stratafix options: ")
    chatMsg("   blshow       : show the blacklist of frames, those ignored by this fixer")
    chatMsg("   bladd <name> : add frame <name> to the blacklist")
    chatMsg("   bldel <name> : remove frame <name> from the blacklist")
  end
end

function addon:FixHelper(lvl, ...)
  local n = select("#",...)
  local changes = 0
  for i=1,n do
    local frame = select(i, ...)
    changes = changes + addon:Fix(frame, lvl)
  end
  return changes
end

function addon:Fix(frame,lvl)
  if not frame:IsShown() or 
     addon.blacklist[frame:GetName() or ""] or
     addon.weak_blacklist[frame] then
    return 0
  end
  local curlvl = frame:GetFrameLevel()
  local changes = 0
  lvl = lvl or curlvl
  if curlvl < lvl then
     curlvl = lvl
     frame:SetFrameLevel(curlvl)
     changes = changes + 1
  end
  changes = changes + addon:FixHelper(curlvl+1, frame:GetChildren())
  return changes
end

function addon:FixAll(silent)
  local changes = 0
  local frame = EnumerateFrames(); -- Get the first frame
  while frame do
    if ( frame:IsToplevel() and frame:IsVisible() ) then
      changes = changes + addon:Fix(frame)
    end
    frame = EnumerateFrames(frame); -- Get the next frame
  end

  if not silent and changes ~= addon.lastchanges then
    chatMsg("Frame strata fixed! ("..changes.." changes)")
  end
  addon.lastchanges = changes
end

hooksecurefunc("ShowUIPanel",function() addon:FixAll(not addon.debug) end)

local upd = 0
function StrataFix_OnUpdate(frame, elapsed)
  if InCombatLockdown() then return end
  upd = upd + elapsed
  if upd < 0.5 then 
    return
  else
    upd = 0
  end
  addon:FixAll(not addon.debug)
end

function StrataFix_OnEvent(frame, event, name, ...)
   if event == "ADDON_LOADED" and name == addonName then
       StrataFixDB = StrataFixDB or {}
       addon.settings = StrataFixDB
       addon:UpdateSettings()
       chatMsg("Revision "..revision.." loaded.")
   elseif event == "PLAYER_REGEN_ENABLED" then 
       -- This event is called when the player exits combat
       frame:SetScript("OnUpdate", StrataFix_OnUpdate);
   elseif event == "PLAYER_REGEN_DISABLED" then
       -- This event is called when we enter combat
       frame:SetScript("OnUpdate", nil);
   end
end


frame:SetScript("OnEvent", StrataFix_OnEvent);
if not InCombatLockdown() then
  frame:SetScript("OnUpdate", StrataFix_OnUpdate);
end
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("PLAYER_REGEN_ENABLED")
frame:RegisterEvent("PLAYER_REGEN_DISABLED")

-- Added bonus: bypass the buggy cancel cinematic confirmation dialog
hooksecurefunc(CinematicFrame.closeDialog,"Show",function()
  CinematicFrame.closeDialog:Hide()
  CinematicFrame_CancelCinematic()
end)

